#!/bin/bash

mkdir -p /var/lib/iqnergy/
FIRSTBOOT_FLAG="/var/lib/iqnergy/firstboot_done.flag"

NODE_TYPE=
LOGIN=
PASSWORD=
NODE_ID=
HANDLE_ERROR=

BRIDGE_CFG=/mnt/data/etc/mosquitto/conf.d/bridge.conf
INITIAL_CFG=/etc/mosquitto/mosquitto.conf

help () {
    echo "To install the controller, set the parameters:"
    echo "-t, --type            TYPE       [ dev | stage | online ]"
    echo "-i, --id              NODE_ID    Some controller id without spaces."
    echo "-u, --mqtt_username   USERNAME   Username on MQTT Server."
    echo "-p, --mqtt_password   PASSWORD   Password on MQTT Server."
    echo
    echo "--handle_error                   Move to the stage in case of an error."
}

find_environment() {
    SOURCE_DIR=/usr/bin/iqnergy/installer

    TIMESTAMP=$(date +"%s")

    USERNAME_LINE=$(grep -a -E ^remote_username ${1})
    PASSWORD_LINE=$(grep -a -E ^remote_password ${1})
    NODE_ID_LINE=$(grep -a -E ^connection ${1})
    SERVER_LINE=$(grep -a -E ^address ${1})

    SERVER_LINE_S1=${SERVER_LINE#* }
    NODE_ID_S1=${NODE_ID_LINE#*_}

    NODE_TYPE=${SERVER_LINE_S1%.iqnergy*}
    LOGIN=${USERNAME_LINE#* }
    PASSWORD=${PASSWORD_LINE#* }
    NODE_ID=${NODE_ID_S1%_*}
}

# Ищем данные входа в bridge.cfg
find_environment ${BRIDGE_CFG}

if  [ -z "$NODE_TYPE" ] || \
    [ -z "$NODE_ID" ] || \
    [ -z "$LOGIN" ] || \
    [ -z "$PASSWORD" ];
then
    # Ищем данные входа в mosquitto.cfg
    find_environment ${INITIAL_CFG}
fi


while [ -n "$1" ]
do
case "$1" in
-t|--type)
    NODE_TYPE="$2"
    shift ;;

-i|--id)
    NODE_ID="$2"
    shift ;;

-u|--mqtt_username)
    LOGIN="$2"
    shift ;;

-p|--mqtt_password)
    PASSWORD=$2
    shift ;;

--upgrade)
    shift ;;

--handle_error)
    HANDLE_ERROR=true ;;

*) help ;;
esac
shift
done


if  [ -z "$NODE_TYPE" ] || \
    [ -z "$NODE_ID" ] || \
    [ -z "$LOGIN" ] || \
    [ -z "$PASSWORD" ];
then
    help;
    exit 1;
fi

# NODE_ID == wirenboard только когда собираем OS в Docker
# В таком случае ID менять смысла нет
if [[ $NODE_ID == "wirenboard" ]]
then
    NODE_ID=$(cat /etc/hostname)
fi

if [ -z "$NODE_TYPE" ] || \
    [ -z "$LOGIN" ] || \
    [ -z "$PASSWORD" ]; then

    # если контроллер потерялся, перенаправляем его на stage
    if [ ! -z "$HANDLE_ERROR" ]; then
        NODE_TYPE=stage
        LOGIN=iqdata
        PASSWORD='NC^R5rfubI7cGF'
        NODE_ID="LOST_${NODE_ID}"
    fi
fi

echo "**************************"
echo "Found bridge configuration:"
echo $NODE_TYPE
echo $LOGIN
echo $NODE_ID
echo "**************************"
echo "**************************"

cp $SOURCE_DIR/mosquitto.conf /etc/mosquitto/mosquitto.conf  || return 1
echo -e "$(eval "echo -e \"`cat \"$SOURCE_DIR/bridge.conf.template\"`\"")" > $BRIDGE_CFG || return 2

CA_CERTIFICATES="/etc/mosquitto/ca_certificates/"
mkdir -p ${CA_CERTIFICATES}
cp /usr/bin/iqnergy/sertificates/ca_*.crt ${CA_CERTIFICATES} || return 3
chmod 644 ${CA_CERTIFICATES}*.crt || return 4

mkdir -p /root/.ssh/
cp "/usr/bin/iqnergy/installer/known_hosts" /root/.ssh/ || return 5

#######################################
# Генерация ключей, для удаленного доступа с сервера по ssh и логирования
if [ "$NODE_ID" != $(cat /etc/hostname) ]; then
    hostname "$NODE_ID"
    echo "$NODE_ID" > /etc/hostname

    rm -f /root/.ssh/id_rsa*
    ssh-keygen -f /root/.ssh/id_rsa -N ""
fi
#######################################
#######################################
